@@ -1,5 +1,5 @@
 package Consul;
-$Consul::VERSION = '0.010';
+$Consul::VERSION = '0.011';
 # ABSTRACT: Client library for consul
 
 use namespace::autoclean;
@@ -59,7 +59,11 @@ sub _prep_response {
 
     my $valid_cb = $args{_valid_cb} || sub { int($_[0]/100) == 2 };
 
-    $content ||= "[no content]" and croak "$status $reason: $content" unless $valid_cb->($status);
+    unless ($valid_cb->($status)) {
+        $content ||= "[no content]";
+        $self->error_cb->("$status $reason: $content");
+        return;
+    }
 
     my $data;
     $data = $json->decode($content) if defined $content && length $content > 0;
@@ -69,8 +73,8 @@ sub _prep_response {
     return ($data, $meta);
 }
 
-has req_cb => ( is => 'lazy', isa => CodeRef );
-sub _build_req_cb {
+has request_cb => ( is => 'lazy', isa => CodeRef );
+sub _build_request_cb {
     sub {
         my ($self, $method, $url, $headers, $content, $cb) = @_;
         my $res = $self->_http->request($method, $url, {
@@ -83,6 +87,13 @@ sub _build_req_cb {
     }
 }
 
+has error_cb => ( is => 'lazy', isa => CodeRef );
+sub _build_error_cb {
+    sub {
+        croak @_;
+    }
+}
+
 sub _api_exec {
     my $resp_cb = $#_ % 2 == 1 && ref $_[$#_] eq 'CODE' ? pop @_ : sub { pop @_ };
     my ($self, $path, $method, %args) = @_;
@@ -90,7 +101,7 @@ sub _api_exec {
     my @r;
     my $cli_cb = delete $args{cb} || sub { @r = @_ };
 
-    $self->req_cb->($self, $self->_prep_request($path, $method, %args), sub {
+    $self->request_cb->($self, $self->_prep_request($path, $method, %args), sub {
         my ($data, $meta) = $self->_prep_response(@_, %args);
         $cli_cb->($resp_cb->($data), $meta);
     });
@@ -199,7 +210,7 @@ method will fail (default: 15).
 
 =item *
 
-C<req_cb>
+C<request_cb>
 
 A callback to an alternative method to make the actual HTTP request. The
 callback is of the form:
@@ -216,11 +227,11 @@ C<$cb> with the returned status, reason, headers and body content.
 
 C<$headers> is a L<Hash::MultiValue>. The returned headers must also be one.
 
-Consul itself provides a default C<req_cb> that uses L<HTTP::Tiny> to make
+Consul itself provides a default C<request_cb> that uses L<HTTP::Tiny> to make
 calls to the server. If you provide one, you should honour the value of the
 C<timeout> argument.
 
-C<req_cb> can be used in conjunction with the C<cb> option to all API method
+C<request_cb> can be used in conjunction with the C<cb> option to all API method
 endpoints to get asynchronous behaviour. It's recommended however that you
 don't use this directly, but rather use a module like L<AnyEvent::Consul> to
 take care of that for you.
@@ -228,6 +239,20 @@ take care of that for you.
 If you just want to use this module to make simple calls to your Consul
 cluster, you can ignore this option entirely.
 
+=item *
+
+C<error_cb>
+
+A callback to an alternative method to handle internal errors (usually HTTP
+errors). The callback is of the form:
+
+    sub {
+        my ($err) = @_;
+        ... output $err ...
+    }
+
+The default callback simply calls C<croak>.
+
 =back
 
 =head1 ENDPOINTS
@@ -286,8 +311,8 @@ some that are common to all methods:
 C<cb>
 
 A callback to call with the results of the method. Without this, the results
-are returned from the method, but only if C<req_cb> is synchronous. If an
-asynchronous C<req_cb> is used without a C<cb> being passed to the method, the
+are returned from the method, but only if C<request_cb> is synchronous. If an
+asynchronous C<request_cb> is used without a C<cb> being passed to the method, the
 method return value is undefined.
 
 If you just want to use this module to make simple calls to your Consul
